using Moq;
using Polly;
using Polly.Timeout;
using System;
using TestingPolly_MockingAndNoOp;
using Xunit;

namespace Tests
{
    public class BusinessLogicTest
    {
        [Fact]
        public void Should_Return_999_When_TimeoutRejectedException_Thrown()
        {
            //Arrange 
            Mock<IErrorProneCode> mockedErrorProneCode = new Mock<IErrorProneCode>();
            mockedErrorProneCode.Setup(e => e.GetSomeNumber()).Returns(0);

            Mock<ISyncPolicy> mockedPolicy = new Mock<ISyncPolicy>();
            mockedPolicy.Setup(p => p.Execute(It.IsAny<Func<int>>())).Throws(new TimeoutRejectedException("Mocked Timeout Exception"));

            BusinessLogic businessLogic = new BusinessLogic(mockedPolicy.Object, mockedErrorProneCode.Object);

            //Act
            // if there is a TimeoutRejectedException in this CallSomeSlowBadCode it will return 999
            int num = businessLogic.CallSomeSlowBadCode();

            //Assert
            Assert.Equal(999, num);
        }

        [Fact]
        public void Should_Return_Odd_When_Retry()
        {
            //Arrange 
            Mock<IErrorProneCode> mockedErrorProneCode = new Mock<IErrorProneCode>();

            Random rnd = new Random(1); // rnd.Next(10) retruns 2, 1, 4 

            mockedErrorProneCode.Setup(e => e.GetSomeNumber()).Returns(() => rnd.Next(10));

            ISyncPolicy<int> policy = Policy.HandleResult<int>(i => i %2 !=1) // retry if the number is not odd.
                .Retry(1);
            OtherBusinessLogic otherBusinessLogic = new OtherBusinessLogic(policy, mockedErrorProneCode.Object);

            //Act
            int num = otherBusinessLogic.CallSomeCodeThatNeedsToBeRetried();

            //Assert
            Assert.Equal(1, num % 2);
        }

        [Fact]
        public void Should_Return_Even_With_NoOp()
        {
            //Arrange 
            Mock<IErrorProneCode> mockedErrorProneCode = new Mock<IErrorProneCode>();

            Random rnd = new Random(1); // rnd.Next(10) retruns 2, 1, 4 

            mockedErrorProneCode.Setup(e => e.GetSomeNumber()).Returns(() => rnd.Next(10));

            ISyncPolicy<int> policy = Policy.NoOp<int>();
            OtherBusinessLogic otherBusinessLogic = new OtherBusinessLogic(policy, mockedErrorProneCode.Object);

            //Act
            int num = otherBusinessLogic.CallSomeCodeThatNeedsToBeRetried();

            //Assert
            Assert.Equal(0, num % 2); //even number
        }
    }
}

